/////////////////////////////////////////////////////////////////////////
//
// File: warjo_tertex.h
//
// Description:
//		MapInter, wj_resource, wj_texture_gen class declaration
//		
//

#ifndef WJ_TERRAIN_TEXGET_H
#define WJ_TERRAIN_TEXGET_H
#include <list>
#include <vector>
#include "warjo_singleton.h"
#include "common.h"
//#include <d3dx8.h>

//******************************************************************
//
// MapInter (map interpolation) class
//
class MapInter {
	inline int Floor(float a)						{ return ((int)a - (a < 0 && a != (int)a)); }

	//the bitmap
	BYTE  *f_stat;
	int m_height,m_width;	
	
	float scale;
public:
	//constructor
	MapInter(BYTE  *map,int h,int w,float s) {m_height=h;m_width=w;f_stat=map;scale=s; }

	//get val from the integer point nx,ny with float parts rx,ry
	float GetVal(int nx,int ny,float rx,float ry) {
		nx+=m_width;
		ny+=m_height;
		nx=nx%m_width;
		ny=ny%m_height;
		
		float p=(float)f_stat[nx+ny*m_width]/255.0f;
		float t=(1-rx)*(1-ry);
		return p*t;
	}

	//get the height at this REAL point
	float GetHeight(float fx,float fy) {
		int nx=Floor(fx);
		int ny=Floor(fy);
		float rx=fx-nx;
		float ry=fy-ny;

		float ft=0;
		ft= (GetVal(nx,ny,rx,ry)+GetVal(nx+1,ny,1-rx,ry)+
			GetVal(nx,ny+1,rx,1-ry)+GetVal(nx+1,ny+1,1-rx,1-ry))*0.25f;

		return  (ft>1.0f)?scale:ft*scale;

	}
};

//******************************************************************
//
// the terrain type
//

struct wj_terrain_type {
	BYTE texID;						//texture ID

	float r,g,b;					//rgb material

	float elevMax,elevMin;			//maximim elevation
	float slopeMax,slopeMin;		//minimum and maximu slope this is found 
	
	float release;					//elevation release
	float sloperelease;				//slope release

	float skew;						//eleavtion skew
	float skewAzimuth;
};

//colection defines
typedef std::list<wj_terrain_type> list_terrain;
typedef std::list<wj_terrain_type>::iterator iter_terrain;

//******************************************************************
//
// the texture type
//
struct wj_texture_type {
	char texture[32];				//texture name
	int  width, height;				//dementions
	BYTE *pb;						//data of the bmp
};

//colection defines
typedef std::vector<wj_texture_type> vector_textures;
typedef std::vector<wj_texture_type>::iterator iter_textures;

//******************************************************************
//
// the color factor type
//
struct color_factor {
	//weight
	float factor;

	//color information
	float r,g,b;
};

//******************************************************************
//
// wj_resource - encapsulates terrains and textures
//
class wj_resource: public Singleton<wj_resource> {

public:
	//get RGB value from a point at tu,tv in texture "id"
	void GetTexturePixel(BYTE id,int tu,int tv,
				float &r,float &g,float &b);

	//Add texture insert the texture in the collection and returns the id
    BYTE AddTexture(const char *szname);

	//texture and terrains collections
	list_terrain terrs;
	vector_textures texs;
};

//global define for resource
#define g_resource wj_resource::GetSingleton()


//******************************************************************
//
// wj_texture_gen - encapsulates terrains and textures
//
class wj_texture_gen {

	//dimensions
	int m_width,m_height;   
	
	//static terrain types
	float slight;				//ambient light
	static VECTOR3 dlight;  //directional light
	static float mapscale;		//mapscale

public:

	static int  pside;

	// the bitmap area
	BYTE	*bitmap;

	//construction/destruction
	wj_texture_gen(int w,int h);
	~wj_texture_gen();

	//renders a pixel color at that height and normal
	DWORD TRender(float h,VECTOR3 &normal,float tu,float tv);

	//mirror the current texture
	void Mirror(BYTE mx,BYTE my);

	//static initialization (for all textures)
	static void Init(void);

	//generate the texutre
	void MakeTexture(BYTE *map,int w,int h,int xf,int yf);
};

#endif